
import os.path
import sys

if getattr(sys, 'frozen', False):
    # if frozen, get embeded file
    cacert = os.path.join(os.path.dirname(sys.executable), 'cacert.pem')
    os.environ["REQUESTS_CA_BUNDLE"] = cacert


import json
import typing
import urllib.error
import urllib.parse
import urllib.request
from email.message import Message


class Response(typing.NamedTuple):
    body: str
    headers: Message
    status: int
    error_count: int = 0

    def json(self) -> typing.Any:
        """
        Decode body's JSON.

        Returns:
            Pythonic representation of the JSON object
        """
        try:
            output = json.loads(self.body)
        except json.JSONDecodeError:
            output = ""
        return output


def request(
        url: str,
        data: dict = None,
        params: dict = None,
        headers: dict = None,
        method: str = "GET",
        data_as_json: bool = True,
        error_count: int = 0,
) -> Response:
    if not url.casefold().startswith("http"):
        raise urllib.error.URLError("Incorrect and possibly insecure protocol in url")
    method = method.upper()
    request_data = None
    headers = headers or {}
    data = data or {}
    params = params or {}
    headers = {"Accept": "application/json", **headers}

    if method == "GET":
        params = {**params, **data}
        data = None

    if params:
        url += "?" + urllib.parse.urlencode(params, doseq=True, safe="/")

    if data:
        if data_as_json:
            request_data = json.dumps(data).encode()
            headers["Content-Type"] = "application/json; charset=UTF-8"
        else:
            request_data = urllib.parse.urlencode(data).encode()

    httprequest = urllib.request.Request(
        url, data=request_data, headers=headers, method=method
    )

    try:
        with urllib.request.urlopen(httprequest) as httpresponse:
            print(httpresponse.headers)
            response = Response(
                headers=httpresponse.headers,
                status=httpresponse.status,
                body=httpresponse.read().decode(
                    httpresponse.headers.get_content_charset("utf-8")
                ),
            )
    except urllib.error.HTTPError as e:
        response = Response(
            body=str(e.reason),
            headers=e.headers,
            status=e.code,
            error_count=error_count + 1,
        )

    return response


server = "https://access4u.eu/"
def deactivate( hardwareId, licenseKey, product_id):

    uri = {
        "api_key": licenseKey,
        "request": "deactivate",
    }
    # The caller should catch errors.
    data = getDataFromActivationServer(uri, hardwareId, product_id)
    if not data.get("success"):
        raise RuntimeError(data['error'])
    deactivated = data.get("deactivated")
    if deactivated:
        message = data['activations_remaining']
        return message
    raise RuntimeError("Deactivation failed")




def getDataFromActivationServer(uri, hardwareId, productId):
    headers = {"Accept": "application/json"}
    uri.update({
        "wc-api": "wc-am-api",
        "product_id": productId,
        "instance": hardwareId,
    })
    print(uri)
    res =  request(
        method="post",
        url=server,
        params=uri,
        headers=headers,
    )
    return res.json()

