
from ctypes import *
from ctypes.wintypes import *
import os
import wx
import addonHandler
import gui
import watchdog
import vision
import ui
from logHandler import log
import config
from gui import nvdaControls
from .dialogs import dialogs


addonHandler.initTranslation()

class USASimpleSettings(wx.Dialog):

	def __init__(self, parent):
		usaInst = None
		veps = vision.handler.getActiveProviderInstances()
		for vep in veps:
			try:
				if vep.getSettings().getId().lower().strip() == "usa":
					usaInst = vep
					break
			except:
				pass
		if not usaInst:
			return

		self.settings = usaInst.getSettings()
		self.dialogs = [dialogs.USACustomColorSchemeDialog, dialogs.USAFGColorSchemeDialog]

		super().__init__(parent, id=wx.ID_ANY, title=_("quick settings"))

		font = wx.Font(18, wx.DEFAULT, wx.NORMAL, wx.BOLD)


		main_sizer = wx.BoxSizer(wx.VERTICAL)

		main_sizer_helper = gui.guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)

		### general USA settings ###
		GeneralGroupText = _("General")
		GeneralGroupSizer = wx.StaticBoxSizer(wx.VERTICAL, self, label=GeneralGroupText)
		GeneralBox = GeneralGroupSizer.GetStaticBox()
		GeneralGroup = gui.guiHelper.BoxSizerHelper(self, sizer=GeneralGroupSizer)
		main_sizer_helper.addItem(GeneralGroup)
		GeneralGroup.addItem(wx.StaticText(GeneralBox, label=_("General settings for USA")))

		trackingButton = GeneralGroup.addItem(wx.Button(GeneralBox, label=_("Tracking")))
		trackingButton.Bind(wx.EVT_BUTTON, self.onTracking)


		# self.bitmapSmoothing = GeneralGroup.addItem(wx.CheckBox(GeneralBox, label=_("Enable bitmap &smoothing to increase magnification quality")))
		# self.bitmapSmoothing.SetValue(self.settings.bitmapSmoothing)
		self.setMagnificationByLevel = GeneralGroup.addItem(wx.CheckBox(GeneralBox, label=_("Let USA set the magnification level by fixed levels")))
		self.setMagnificationByLevel.SetValue(self.settings.setMagnificationByLevel)

		stepLevelComboboxText = _("step level")
		self.stepLevelStrings = [ii.displayName for ii in list(self.settings.availableSteplevels.values())]
		self.stepLevelCombobox = GeneralGroup.addLabeledControl(stepLevelComboboxText, wx.Choice, choices=self.stepLevelStrings)
		self.stepLevelCombobox.SetSelection( self.stepLevelStrings.index(str(self.settings.stepLevel)) )

		### settings for Object Reader ###
		objectReaderGroupText = _("Object Reader")
		objectReaderGroupSizer = wx.StaticBoxSizer(wx.VERTICAL, self, label=objectReaderGroupText)
		objectReaderBox = objectReaderGroupSizer.GetStaticBox()
		objectReaderGroup = gui.guiHelper.BoxSizerHelper(self, sizer=objectReaderGroupSizer)
		main_sizer_helper.addItem(objectReaderGroup)
		objectReaderGroup.addItem(wx.StaticText(objectReaderBox, label=_("Enable users to apply continuous reading for objects below the mouse.")))
		self.objectReaderMode = objectReaderGroup.addItem(wx.CheckBox(objectReaderBox, label=_("Activate Object Reader Mode")))
		self.objectReaderMode.SetValue(self.settings.objectReaderMode)
		self.followWord = objectReaderGroup.addItem(
			wx.CheckBox(objectReaderBox, label=_("Enable following of words in continuous reading mode.")))
		self.followWord.SetValue(self.settings.followWord)
		self.followWordRect = objectReaderGroup.addItem(
			wx.CheckBox(objectReaderBox, label=_("Enable highlighting of words in continuous reading mode.")))
		self.followWordRect.SetValue(self.settings.followWordRect)
		self.followWordJumpCaret = objectReaderGroup.addItem(
			wx.CheckBox(objectReaderBox, label=_("Let the caret jump with the read words in continuous reading mode.")))
		self.followWordJumpCaret.SetValue(self.settings.followWordJumpCaret)

		aboutObjectReader = objectReaderGroup.addItem(wx.Button(objectReaderBox, label=_("About")))
		aboutObjectReader.Bind(wx.EVT_BUTTON, self.aboutObjectReader)



		### settings for remote capabilities ###
		remoteGroupText = _("Remote")
		remoteGroupSizer = wx.StaticBoxSizer(wx.VERTICAL, self, label=remoteGroupText)
		remoteBox = remoteGroupSizer.GetStaticBox()
		remoterGroup = gui.guiHelper.BoxSizerHelper(self, sizer=remoteGroupSizer)
		main_sizer_helper.addItem(remoterGroup)
		remoterGroup.addItem(wx.StaticText(remoteBox, label=_("Enable users to run USA in combination with remote desktop sessions. Works in combination with Unicorn.")))
		self.isRemote = remoterGroup.addItem(wx.CheckBox(remoteBox, label=_("Run USA in remote Mode (with unicorn)")))
		self.isRemote.SetValue(self.settings.isRemote)
		aboutRemote = remoterGroup.addItem(wx.Button(remoteBox, label=_("About")))
		aboutRemote.Bind(wx.EVT_BUTTON, self.aboutRemote)

		### settings for color shcemes capabilities ###
		colorGroupText = _("Color Schemes")
		colorGroupSizer = wx.StaticBoxSizer(wx.VERTICAL, self, label=colorGroupText)
		colorBox = colorGroupSizer.GetStaticBox()
		colorGroup = gui.guiHelper.BoxSizerHelper(self, sizer=colorGroupSizer)
		main_sizer_helper.addItem(colorGroup)
		colorGroup.addItem(wx.StaticText(colorBox, label=_("Settings for color schemes in USA")))

		modifierBoxLabel = _("&Select Colorschemes")
		colorSchemeDict = self.settings._get_allColorschemes()

		self.modifierChoices = [colorSchemeDict[key].displayName for key in colorSchemeDict]
		self.modifierList = colorGroup.addLabeledControl(modifierBoxLabel, nvdaControls.CustomCheckListBox,
															   choices=self.modifierChoices)
		checkedItems = []
		log.debug(f"USA: {self.modifierChoices}")
		if self.settings.conf.get("selectedDefault") == 'True':
			checkedItems.append(self.modifierChoices.index(colorSchemeDict["default"].displayName))
		if self.settings.conf.get("selectedInverted") == 'True':
			checkedItems.append(self.modifierChoices.index(colorSchemeDict["inverted"].displayName))
		if self.settings.conf.get("selectedGray") == 'True':
			checkedItems.append(self.modifierChoices.index(colorSchemeDict["gray"].displayName))
		if self.settings.conf.get("selectedHighContrast") == 'True':
			checkedItems.append(self.modifierChoices.index(colorSchemeDict["highContrast"].displayName))
		if self.settings.conf.get("selectedInvertedHighContrast") == 'True':
			checkedItems.append(self.modifierChoices.index(colorSchemeDict["invertedHighContrast"].displayName))
		if self.settings.conf.get("selectedBlueLight") == 'True':
			checkedItems.append(self.modifierChoices.index(colorSchemeDict["blueLight"].displayName))
		if self.settings.conf.get("selectedCustom") == 'True':
			checkedItems.append(self.modifierChoices.index(colorSchemeDict["custom"].displayName))

		self.modifierList.CheckedItems = checkedItems
		self.modifierList.Select(0)

		self.colorButton = colorGroup.addItem(
			wx.Button(colorBox, label=_("Manage custom color sch&eme...")))
		self.colorButton.Bind(wx.EVT_BUTTON, self.onColor)
		self.colorButton.Enable(bool(self.settings.supportedSettings))

		# button for fg depending color schemes
		self.fgColorButton = colorGroup.addItem(
			wx.Button(colorBox, label=_("Manage App-depending color schemes")))
		self.fgColorButton.Bind(wx.EVT_BUTTON, self.onFGColor)
		self.fgColorButton.Enable(bool(self.settings.supportedSettings))

		main_sizer_helper.addDialogDismissButtons(self.CreateButtonSizer(wx.OK | wx.CANCEL))
		self.Bind(wx.EVT_BUTTON, self.on_ok, id=wx.ID_OK)
		main_sizer.Add(main_sizer_helper.sizer, border = gui.guiHelper.BORDER_FOR_DIALOGS, flag=wx.ALL)
		main_sizer.Fit(self)
		self.SetSizer(main_sizer)
		self.Center(wx.BOTH )
		self.Show()

	def onTracking(self, evt):
		with trackingSettings(self) as dlg:
			dlg.ShowModal()



	def aboutObjectReader(self,evt):
		message = _("Object Reader allows the user to start reading the text that is below the cursor. "
					"With NVDA+mouseclick the user can activate the continuous reading mode of NVDA starting with the text below the cursor. "
					"Enabling this function will do the following: \n"
					"-The speech in NVDA is automatically disabled (but can be enabled with NVDA+S)\n"
					"-When this is used in browsers, USA will automatically switch between focus and browsemode when NVDA+alt+R is pressed\n"
					"-An advanced setting for forcing UIA elements in Word and Outlook is enabled in NVDA\n"
					"Note: this functionality does not work well with PDFs!")
		gui.messageBox(message, _("about Object Reader"))

	def aboutRemote(self, evt):
		message = _("To work with USA in remote sessions unicornDVC is used. With this setup the USA on the local machine applies the magnification while it is able to receive"
					" the necessary information from the USA of the remote machine. ")
		gui.messageBox(message, _("about USA Remote"))


	def on_ok(self, evt):
		self.settings.objectReaderMode = self.objectReaderMode.GetValue()
		self.settings.followWord = self.followWord.GetValue()
		self.settings.followWordRect = self.followWordRect.GetValue()
		self.settings.followWordJumpCaret = self.followWordJumpCaret.GetValue()

		if self.settings.objectReaderMode:


			config.conf["UIA"]["allowInMSWord"] = 3 # always
		else:
			config.conf["UIA"]["allowInMSWord"] = 0 # default

		if self.isRemote.GetValue() and not self.settings.remoteHandler:
			gui.messageBox(_("Unicorn NVDA addon cannot be found! USA cannot run in remote if Unicorn is not active!"))
			self.settings.isRemote = False
		else:
			self.settings.isRemote = self.isRemote.GetValue()

		stepLevelIndex = self.stepLevelCombobox.CurrentSelection
		stepLevelValue = float(self.stepLevelStrings[stepLevelIndex])
		self.settings.stepLevel = stepLevelValue

		# self.settings.bitmapSmoothing = self.bitmapSmoothing.GetValue()
		self.settings.setMagnificationByLevel = self.setMagnificationByLevel.GetValue()
		colorSchemeDict = self.settings._get_allColorschemes()
		self.settings.conf["selectedDefault"] = str(
			self.modifierList.IsChecked(self.modifierChoices.index(colorSchemeDict["default"].displayName)))
		self.settings.conf["selectedInverted"] = str(
			self.modifierList.IsChecked(self.modifierChoices.index(colorSchemeDict["inverted"].displayName)))
		self.settings.conf["selectedGray"] = str(
			self.modifierList.IsChecked(self.modifierChoices.index(colorSchemeDict["gray"].displayName)))
		self.settings.conf["selectedHighContrast"] = str(
			self.modifierList.IsChecked(self.modifierChoices.index(colorSchemeDict["highContrast"].displayName)))
		self.settings.conf["selectedInvertedHighContrast"] = str(self.modifierList.IsChecked(
			self.modifierChoices.index(colorSchemeDict["invertedHighContrast"].displayName)))
		self.settings.conf["selectedBlueLight"] = str(
			self.modifierList.IsChecked(self.modifierChoices.index(colorSchemeDict["blueLight"].displayName)))
		self.settings.conf["selectedCustom"] = str(
			self.modifierList.IsChecked(self.modifierChoices.index(colorSchemeDict["custom"].displayName)))

		self.settings.saveSettings()
		evt.Skip()
		self.Close()
		self.DestroyLater()

	def onColor(self, evt):
		colorDialog = self.dialogs[0]
		with colorDialog(self, self.settings) as dlg:
			dlg.ShowModal()

	def onFGColor(self, evt):
		fgColorDialog = self.dialogs[1]
		with fgColorDialog(self, self.settings) as dlg:
			dlg.ShowModal()





class trackingGroup:

	def __init__(self, parent, settings, name, checkboxLabelList, main_sizer_helper, attrNameBound, attrNameFollowList ):
		self.settings = settings
		self.attrNameBound = attrNameBound
		self.attrNameFollowList = attrNameFollowList
		boundaryLevelsText = _("boundary")
		self.boundaryLevelChoices = [ii.displayName for ii in list(self.settings.availableFocusboundarys.values())]
		self.boundaryLevelValues = [ii.id for ii in list(self.settings.availableFocusboundarys.values())]
		FocusGroupText = name
		FocusGroupSizer = wx.StaticBoxSizer(wx.VERTICAL, parent, label=FocusGroupText)
		FocusBox = FocusGroupSizer.GetStaticBox()
		FocusGroup = gui.guiHelper.BoxSizerHelper(parent, sizer=FocusGroupSizer)
		main_sizer_helper.addItem(FocusGroup)

		self.checkboxes = []
		for ii in range(len(attrNameFollowList)) :
			focusBoundaryCheckbox = FocusGroup.addItem(wx.CheckBox(FocusBox, label=checkboxLabelList[ii]))
			focusBoundaryCheckbox.SetValue(getattr(self.settings, attrNameFollowList[ii]))
			self.checkboxes.append(focusBoundaryCheckbox)

		self.focusBoundaryBox = FocusGroup.addLabeledControl(boundaryLevelsText, wx.Choice,
															 choices=self.boundaryLevelChoices)
		self.focusBoundaryBox.SetSelection(self.boundaryLevelChoices.index(
			str(self.settings.availableFocusboundarys[tuple(getattr(self.settings, attrNameBound))].displayName)))

	def onSave(self):
		focusBoundaryIndex = self.focusBoundaryBox.CurrentSelection
		focusBoundaryValue = self.boundaryLevelValues[focusBoundaryIndex]
		setattr(self.settings, self.attrNameBound,focusBoundaryValue)

		for ii in range(len(self.checkboxes)):
			setattr(self.settings, self.attrNameFollowList[ii], self.checkboxes[ii].IsChecked())


class trackingSettings(wx.Dialog):

	def __init__(self, parent):
		usaInst = None
		veps = vision.handler.getActiveProviderInstances()
		for vep in veps:
			try:
				if vep.getSettings().getId().lower().strip() == "usa":
					usaInst = vep
					break
			except:
				pass
		if not usaInst:
			return

		self.settings = usaInst.getSettings()

		super().__init__(parent, id=wx.ID_ANY, title=_("tracking settings"))

		font = wx.Font(18, wx.DEFAULT, wx.NORMAL, wx.BOLD)


		main_sizer = wx.BoxSizer(wx.VERTICAL)
		message = _("tracking settings for USA")
		titleText = wx.StaticText(self, label=message)
		main_sizer.Add(titleText)
		titleText.SetFont(font)
		main_sizer_helper = gui.guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)



		self.focusTrackingGroup = trackingGroup(self, self.settings, _("Focus"), [_("Follow &focus")], main_sizer_helper , "focusBoundary", ["trackToFocus"])
		self.mouseTrackingGroup = trackingGroup(self, self.settings, _("Mouse"), [_("Follow &mouse")], main_sizer_helper,
											"mouseBoundary", ["trackToMouse"])
		self.caretTrackingGroup = trackingGroup(self, self.settings, _("Caret"), [_("Follow system &caret")], main_sizer_helper,
												"caretBoundary", ["trackToCaret"])
		self.reviewGroup = trackingGroup(self, self.settings, _("Browse Mode"), [_("Follow &browse mode cursor")],
												main_sizer_helper,
												"browseModeBoundary", ["trackToBrowseMode"])


		main_sizer_helper.addDialogDismissButtons(self.CreateButtonSizer(wx.OK | wx.CANCEL))
		self.Bind(wx.EVT_BUTTON, self.on_ok, id=wx.ID_OK)
		main_sizer.Add(main_sizer_helper.sizer, border = gui.guiHelper.BORDER_FOR_DIALOGS, flag=wx.ALL)
		main_sizer.Fit(self)
		self.SetSizer(main_sizer)
		self.Center(wx.BOTH )
		self.Show()





	def on_ok(self, evt):



		self.focusTrackingGroup.onSave()
		self.mouseTrackingGroup.onSave()
		self.caretTrackingGroup.onSave()
		self.reviewGroup.onSave()


		self.settings.saveSettings()
		evt.Skip()
		self.Close()
		self.DestroyLater()




