


from ctypes import *
from ctypes.wintypes import *
import time
import os
import os.path
import winreg
from logHandler import log

usaVersions = [
    "USA2024",
    "USA2025",
    "USA2026",
    "USA2027"
]
productID = 1356

NO_DEMO_FOUND = 0
DEMO_FOUND = 1
DEMO_OVER = 2
DEMO_INACTIVE = 3


class demoHandler:
    status = []
    def __init__(self):
        self.status = self.loadFromRegistry()

    def DemoOver(self, version, id_Database):
        return [DEMO_OVER, version, id_Database]

    def DemoUpdate(self, version, id_Database, timeStamp):
        return [DEMO_FOUND, version, id_Database, timeStamp]

    def noDemoFound(self):
        return [NO_DEMO_FOUND,]

    def inactiveDemoFound(self, version, id_Database):
        return [DEMO_INACTIVE, version, id_Database]

    def loadFromRegistry(self):
        for usaVersion in usaVersions:
            regPath = os.path.join("SOFTWARE", "ACCESS4U", usaVersion)
            try:
                with winreg.OpenKeyEx(
                        winreg.HKEY_CURRENT_USER,
                        regPath,
                        0,
                        winreg.KEY_READ | winreg.KEY_WOW64_32KEY
                ) as key:
                    log.debugWarning(f"loading licid")
                    licenseKey = winreg.QueryValueEx(key, "LicId")[0]
                    log.debugWarning(f"got licID {licenseKey}")
                    if licenseKey == "DEMO":
                        try:
                            hash = winreg.QueryValueEx(key, "AcId")[0]
                            timeStamp = int(hash[:8], 16)
                        except:
                            # no hash, so it is an old demo
                            timeStamp = 0
                        id_Database = winreg.QueryValueEx(key, "demoID")[0]
                        currentTime = int(time.time())
                        if currentTime > timeStamp:
                            return self.DemoOver(usaVersion, id_Database)
                        else:
                            return self.DemoUpdate(usaVersion, id_Database, timeStamp)
                        
            except Exception as e:
                log.debugWarning(f"could not load license from registry for {usaVersion}\n{e}")
                try:
                    with winreg.OpenKeyEx(
                        winreg.HKEY_CURRENT_USER,
                        regPath,
                        0,
                        winreg.KEY_READ | winreg.KEY_WOW64_32KEY
                    ) as key:
                        # try to see if there is a database id for the demo anyways. This can be the case when the user had a demo and after that activated it
                        id_Database = winreg.QueryValueEx(key, "demoID")[0]
                        return self.inactiveDemoFound(usaVersion, id_Database)
                except:
                    pass

                pass

        return self.noDemoFound()

handler = demoHandler()